package be.jeffcheasey88.peeratcode;

import static be.jeffcheasey88.peeratcode.framework.RequestType.OPTIONS;

import java.util.regex.Matcher;

import be.jeffcheasey88.peeratcode.framework.HttpReader;
import be.jeffcheasey88.peeratcode.framework.HttpWriter;
import be.jeffcheasey88.peeratcode.framework.Locker;
import be.jeffcheasey88.peeratcode.framework.Response;
import be.jeffcheasey88.peeratcode.framework.Route;
import be.jeffcheasey88.peeratcode.framework.Router;
import be.jeffcheasey88.peeratcode.framework.User;
import be.jeffcheasey88.peeratcode.model.Completion;
import be.jeffcheasey88.peeratcode.model.Group;
import be.jeffcheasey88.peeratcode.repository.DatabaseRepository;
import be.jeffcheasey88.peeratcode.routes.BadgeDetails;
import be.jeffcheasey88.peeratcode.routes.ChapterElement;
import be.jeffcheasey88.peeratcode.routes.ChapterList;
import be.jeffcheasey88.peeratcode.routes.DynamicLeaderboard;
import be.jeffcheasey88.peeratcode.routes.Leaderboard;
import be.jeffcheasey88.peeratcode.routes.Login;
import be.jeffcheasey88.peeratcode.routes.PlayerDetails;
import be.jeffcheasey88.peeratcode.routes.PuzzleElement;
import be.jeffcheasey88.peeratcode.routes.PuzzleResponse;
import be.jeffcheasey88.peeratcode.routes.Register;
import be.jeffcheasey88.peeratcode.routes.Result;
import be.jeffcheasey88.peeratcode.routes.groups.GroupCreate;
import be.jeffcheasey88.peeratcode.routes.groups.GroupJoin;
import be.jeffcheasey88.peeratcode.routes.groups.GroupList;
import be.jeffcheasey88.peeratcode.routes.groups.GroupQuit;

public class Main{
	public static void main(String[] args) throws Exception{
		Configuration config = new Configuration("config.txt");
		config.load();

		Class.forName("com.mysql.cj.jdbc.Driver");

		DatabaseRepository repo = new DatabaseRepository(config);
		Router router = new Router(config.getTokenIssuer(),
				config.getTokenExpiration());

		router.setDefault((matcher, user, reader, writer) -> {
			writer.response(404, "Access-Control-Allow-Origin: *");
			writer.write("404 not Found.\n");
			writer.flush();
			writer.close();
		});
		
		router.register(new Response(){
			@Route(path = "^(.*)$", type = OPTIONS)
			@Override
			public void exec(Matcher matcher, User user, HttpReader reader, HttpWriter writer) throws Exception {
				writer.response(200, "Access-Control-Allow-Origin: *",
						"Access-Control-Allow-Methods: *", "Access-Control-Allow-Headers: *");
			}
		});
		
		initRoutes(router, repo, config);

//		RouteExtracter extracter = new RouteExtracter(router);
//		extracter.extract();
		
		if(config.useSsl()) router.configureSSL(config.getSslKeystore(), config.getSslKeystorePasswd());
		router.listen(config.getTcpPort(), config.useSsl());
	}

	private static void initRoutes(Router router, DatabaseRepository repo, Configuration config){
		router.register(new ChapterElement(repo));
		router.register(new ChapterList(repo));
		router.register(new PuzzleElement(repo));
		router.register(new Register(repo, router, config.getUsersFiles()));
		router.register(new Login(repo, router));
		router.register(new Result(repo));
		router.register(new Leaderboard(repo));
		router.register(new PlayerDetails(repo));
		router.register(new BadgeDetails(repo));
		
		Locker<Group> groupLock = new Locker<>();
		router.register(new GroupCreate(repo, groupLock, config.getGroupJoinMinutes()));
		
		DynamicLeaderboard dlb = new DynamicLeaderboard(repo);
		router.register(dlb);
		
		Locker<Completion> leaderboard = dlb.getLocker();
		
		router.register(new PuzzleResponse(repo, config.getUsersFiles(), leaderboard));
		router.register(new GroupList(repo));
		router.register(new GroupJoin(repo, config.getGroupJoinMinutes(), config.getGroupQuitMinutes(), leaderboard));
		router.register(new GroupQuit(repo, config.getGroupJoinMinutes(), leaderboard));
		
//		Bot bot = new Bot(config, repo, groupLock);
//		bot.start();
	}
}
